/*
 * File: pass.cc
 *  date: July, 1998
 * Author: Yukio Hashimoto
   Thanks to Marc Greis (greis@cs.uni-bonn.de)
 *
 */

#include "agent.h"
#include "tclcl.h"
#include "packet.h"
#include "ip.h"
#include "sar.h"
#include "rtp.h"

 
class PassAgent : public Agent {
 public:
  PassAgent();
  int command(int argc, const char*const* argv);
  void recv(Packet*, Handler*);
protected:
  int off_sar_;
  NsObject* dst;
  int off_rtp_;

};
 



static class PassClass : public TclClass {
public:
  PassClass() : TclClass("Agent/Pass") {}
  TclObject* create(int, const char*const*) {
    return (new PassAgent());
  }
} class_pass;


PassAgent::PassAgent() : Agent(PT_CBR)
{
  bind("off_sar_", &off_sar_); // important to access sar packets
  bind("off_rtp_", &off_rtp_);
}


int PassAgent::command(int argc, const char*const* argv)
{
  if (argc == 3) {
    if (strcmp(argv[1], "dst") == 0) {
      Agent* a = (Agent*)TclObject::lookup(argv[2]);
      dst=a;

      // command has been processed */
      return (TCL_OK);
    }
  }
  // If the command hasn't been processed by PassAgent()::command,
  // call the command() function for the base class
  return (Agent::command(argc, argv));
}


void PassAgent::recv(Packet* pkt, Handler*)
{
  // Access the IP header for the received packet:
  hdr_cmn* hdrcmn = (hdr_cmn*)pkt->access(off_cmn_);
  
  if(hdrcmn->ptype() == PT_CBR){  // to lower layer
    dst->recv(pkt,0);
  }  else {                       // from lower layer

    // Access the Pass header for the received packet:
     hdr_sar* hdra = (hdr_sar*)pkt->access(off_sar_);
    
    Packet* p = allocpkt();  // default CBR
    hdr_cmn* hdrcmn2 = (hdr_cmn*)p->access(off_cmn_);
    
    hdrcmn2->size()=hdra->pkt_size;

    hdr_ip* hdrip = (hdr_ip*)pkt->access(off_ip_);
    *hdrip= hdra->IP;


    hdr_rtp* rh = (hdr_rtp*)p->access(off_rtp_);
    hdr_rtp* rh_from = (hdr_rtp*)pkt->access(off_rtp_);
    *rh = *rh_from;  // copy rtp info

    

    send(p,0);
    
     

    Packet::free(pkt);  
  }

}


